local opt = {}

-- Database options
opt["itemdata"] = true       -- Use item data
opt["spelldata"] = true      -- Use spell data

-- Chat options
opt["chat"] = true           -- Translate chat links
opt["mylinks"] = true        -- Translate own links

-- Race and Class options
opt["raceclass"] = true      -- Translate Race and Class of the player

-- Castbar Options
opt["castbar"] = true        -- Translate own castbar

-- Combat log options
opt["combatlog"] = true      -- Translate the combat log. Affect all addons using combat logs (Recount, SCT etc.)

-- Frame options
opt["frameloot"] = true      -- Tranlate items in loot frame
opt["frameprof"] = true      -- Translate the professions UI
opt["framequest"] = true     -- Translate items in quest window
opt["framevendor"] = true    -- Translate items in vendor frame 

-- Tooltip options
opt["tipitem"] = true        -- Translate item in tooltip
opt["tipspell"] = true       -- Translate spell name in tooltip
opt["tiptalent"] = true      -- Translate talent names in tooltip
opt["tipformat"] = 0         -- Format: 0 - english names only, 1 - english name (russian in brackets), 2 - russian name  (english in brackets). (default: 0)
opt["tipcolor"] = "fffe8b"   -- Brackets translation color rgb. (по умолчанию "fffe8b")
opt["keydown"] = "alt"       -- Remove translation with modifier key. Possible  - "alt", "ctrl", "shift"

-- GlobalStrings options
opt["globalstr"] = true      -- Use GlobalStrings.lua from the english client. May cause interface errors

-- Cannot translate without item data
if (opt["itemdata"] == false) then
	opt["frameloot"] = false
	opt["frameprof"] = false
	opt["framequest"] = false
	opt["framevendor"] = false
	opt["tipitem"] = false
end

-- Cannot translate without spell data
if (opt["spelldata"] == false) then
	opt["castbar"] = false
	opt["combatlog"] = false
	opt["tipspell"] = false
	opt["tiptalent"] = false
end

-- Key modifers
local dl_iskeyDown
if (opt["keydown"] == "alt") then
	dl_iskeyDown = IsAltKeyDown
elseif (opt["keydown"] == "ctrl") then
	dl_iskeyDown = IsControlKeyDown
elseif (opt["keydown"] == "shift") then
	dl_iskeyDown = IsShiftKeyDown
else
	dl_iskeyDown = function() return false end
end

-- Register library, load addons, set options
local TipHooker = LibStub:GetLibrary("LibTipHooker-1.1")
Delocalizator = LibStub("AceAddon-3.0"):NewAddon("Delocalizator", "AceConsole-3.0", "AceHook-3.0", "AceEvent-3.0", "AceTimer-3.0")
Delocalizator.opt = opt

local linkPattern = "(|c%x+|H.-|h%[.-%]|h|r)"
local frame_handlers = {}

function Delocalizator:OnInitialize()
	-- Merge item data
	if (opt["itemdata"] == true) then
		for id in pairs(Delocalizator.itemdata_part1) do
			Delocalizator.itemdata[id] = Delocalizator.itemdata_part1[id]
		end
		for id in pairs(Delocalizator.itemdata_part2) do
			Delocalizator.itemdata[id] = Delocalizator.itemdata_part2[id]
		end
	end

	-- Merge spell data
	if (opt["spelldata"] == true) then
		for id in pairs(Delocalizator.spelldata_part1) do
			Delocalizator.spelldata[id] = Delocalizator.spelldata_part1[id]
		end
		for id in pairs(Delocalizator.spelldata_part2) do
			Delocalizator.spelldata[id] = Delocalizator.spelldata_part2[id]
		end
	end

	-- Hooks
	if opt["chat"]        then self:RawHook("ChatFrame_OnEvent", true) end               -- Chatframe hook
	if opt["castbar"]     then self:RawHook("UnitCastingInfo", true) end                 -- Castbar hook
	if opt["framevendor"] then self:RawHook("GetMerchantItemInfo", true) end             -- Vendor frame hook
	if opt["frameprof"]   then self:RawHook("GetTradeSkillReagentInfo", true) end        -- Tradeskill frame hook
	if opt["frameprof"]   then self:RawHook("GetTradeSkillInfo", true) end               -- Tradeskill frame hook
	if opt["frameloot"]   then self:RawHook("GetLootSlotInfo", true) end                 -- Lootframe hook

	-- Quest frames hook
	if opt["framequest"] then
		self:RawHook("GetQuestItemInfo", true)
		self:RawHook("GetQuestLogRewardInfo", true)
		self:RawHook("GetQuestLogChoiceInfo", true)
	end
	
	-- Tooltip hook
	if opt["tipitem"]    then TipHooker:Hook(self.ProcessItem,   "item")   end
	if opt["tipspell"]   then TipHooker:Hook(self.ProcessSpell,  "spell")  end

	-- Talent's hook
	if opt["tiptalent"]  then
		-- Only if character's level >= 10
		if UnitLevel("player") >= 10 then
			TipHooker:Hook(self.ProcessTalent, "talent")
		end
	end

	TipHooker:Hook(self.ProcessAction, "action")
	
	-- Editbox hook
	if opt["mylinks"] then
		local hk1 = ChatFrame1EditBox.Insert
		ChatFrame1EditBox.Insert = function(self, text)
			local engLink = Delocalizator.StringToEnglish(text)
			if engLink then text = engLink end
			return hk1(self, text)
		end
	end
	
	-- UnitClass hook
	if opt["raceclass"] then
		--self:RawHook("UnitClass", true)
		--self:RawHook("UnitRace", true)
	end
	
	-- delay for combat events for other addons to finish loading
	if opt["combatlog"] then Delocalizator:ScheduleTimer("CombatLogHook", 5) end
	
end

---------------------
-- Race&Class hook --
---------------------
function Delocalizator:UnitClass(unit, name)
	local locName, engName = self.hooks["UnitClass"](unit)
	
	if engName == "WARRIOR" then
		locName = "Warrior"
	elseif engName == "PRIEST" then
		locName = "Priest"
	elseif engName == "MAGE" then
		locName = "Mage"
	elseif engName == "SHAMAN" then
		locName = "Shaman"
	elseif engName == "PALADIN" then
		locName = "Paladin"
	elseif engName == "WARLOCK" then
		locName = "Warlock"
	elseif engName == "DRUID" then
		locName = "Druid"
	elseif engName == "ROGUE" then
		locName = "Rogue"
	elseif engName == "HUNTER" then
		locName = "Hunter"
	elseif engName == "DEATHKNIGHT" then
		locName = "Death Knight"
	end
	
	return locName, engName
end
		
function Delocalizator:UnitRace(unit)
	local race, enRace = self.hooks["UnitRace"](unit)
	
	if enRace == "BloodElf" then
		race = "Blood Elf"
	elseif enRace == "Draenei" then
		race = "Draenei"
	elseif enRace == "Dwarf" then
		race = "Dwarf"
	elseif enRace == "Gnome" then
		race = "Gnome"
	elseif enRace == "Human" then
		race = "Human"
	elseif enRace == "NightElf" then
		race = "Night Elf"
	elseif enRace == "Orc" then
		race = "Orc"
	elseif enRace == "Tauren" then
		race = "Tauren"
	elseif enRace == "Troll" then
		race = "Troll"
	elseif enRace == "Scourge" then
		race = "Undead"
	elseif enRace == "Goblin" then
		race = "Goblin"
	elseif enRace == "Worgen" then
		race = "Worgen"
	end
	
	return race, enRace
end
		
-----------------------
-- Vendor frame hook --
-----------------------
function Delocalizator:GetMerchantItemInfo(index)

	local link = GetMerchantItemLink(index)
	local itemName, texture, price, quantity, numAvailable, isUsable, extendedCost = self.hooks["GetMerchantItemInfo"](index)
	
	if link then 
		local _, _, itemString, _ = string.find(link, "|c%x+|H(.-)|h%[(.-)%]|h|r")
		local name = Delocalizator:GetTranslation(itemString)
		if name then itemName = name end
	end
	
	return itemName, texture, price, quantity, numAvailable, isUsable, extendedCost

end

----------------------
-- Quest frame hook --
----------------------
function Delocalizator:GetQuestItemInfo(type, itemNum)
		
	local link = GetQuestItemLink(type, itemNum)
	local itemName, texture, numItems, quality, isUsable = self.hooks["GetQuestItemInfo"](type, itemNum)
	
	if link then 
		local _, _, itemString, _ = string.find(link, "|c%x+|H(.-)|h%[(.-)%]|h|r")
		local name = Delocalizator:GetTranslation(itemString)
		if name then itemName = name end
	end
	
	return itemName, texture, numItems, quality, isUsable
	
end

function Delocalizator:GetQuestLogRewardInfo(itemNum)
	
	local link = GetQuestLogItemLink("reward", itemNum)	
	local itemName, texture, numItems, quality, isUsable = self.hooks["GetQuestLogRewardInfo"](itemNum)
	
	if link then 
		local _, _, itemString, _ = string.find(link, "|c%x+|H(.-)|h%[(.-)%]|h|r")
		local name = Delocalizator:GetTranslation(itemString)
		if name then itemName = name end
	end
	
	return itemName, texture, numItems, quality, isUsable
	
end

function Delocalizator:GetQuestLogChoiceInfo(itemNum)
		
	local link = GetQuestLogItemLink("choice", itemNum)
	local itemName, texture, numItems, quality, isUsable = self.hooks["GetQuestLogChoiceInfo"](itemNum)
	
	if link then 
		local _, _, itemString, _ = string.find(link, "|c%x+|H(.-)|h%[(.-)%]|h|r")
		local name = Delocalizator:GetTranslation(itemString)
		if name then itemName = name end
	end
	
	return itemName, texture, numItems, quality, isUsable
	
end

----------------------------
-- Tradeskills frame hook --
----------------------------
function Delocalizator:GetTradeSkillInfo(skillIndex)

	local link = GetTradeSkillRecipeLink(skillIndex);
	local skillName, skillType, numAvailable, isExpanded, altVerb, numSkillUps = self.hooks["GetTradeSkillInfo"](skillIndex)

	if link then 
		local _, _, itemString, _ = string.find(link, "|c%x+|H(.-)|h%[(.-)%]|h|r")
		local name = Delocalizator:GetTranslation(itemString)
		if name then skillName = name end
	end
	
	return skillName, skillType, numAvailable, isExpanded, altVerb, numSkillUps
end

function Delocalizator:GetTradeSkillReagentInfo(tradeSkillRecipeId, reagentId)

	local link = GetTradeSkillReagentItemLink(tradeSkillRecipeId, reagentId)
	local reagentName, reagentTexture, reagentCount, playerReagentCount = self.hooks["GetTradeSkillReagentInfo"](tradeSkillRecipeId, reagentId)
	
	if link then 
		local _, _, itemString, _ = string.find(link, "|c%x+|H(.-)|h%[(.-)%]|h|r")
		local name = Delocalizator:GetTranslation(itemString)
		if name then reagentName = name end
	end
	
	return reagentName, reagentTexture, reagentCount, playerReagentCount
end

---------------------
-- Loot frame hook --
---------------------
function Delocalizator:GetLootSlotInfo(index)

	local link = GetLootSlotLink(index)
	local lootIcon, lootName, lootQuantity, rarity, locked = self.hooks["GetLootSlotInfo"](index)
	
	if link then 
		local _, _, itemString, _ = string.find(link, "|c%x+|H(.-)|h%[(.-)%]|h|r")
		local name = Delocalizator:GetTranslation(itemString)
		if name then lootName = name end
	end
	
	return lootIcon, lootName, lootQuantity, rarity, locked
	
end

------------------
-- Castbar hook --
------------------
function Delocalizator:UnitCastingInfo(unit)

	local spell, rank, displayName, icon, startTime, endTime, isTradeSkill, castID, interrupt = self.hooks["UnitCastingInfo"](unit)
	local link, name
	
	if (spell) then
		if (rank) then
			link = GetSpellLink(spell, rank)
		else
			link = GetSpellLink(spell)
		end
	end
	
	if (link) then
		local _, _, itemString, itemName = string.find(link, "|c%x+|H(.-)|h%[(.-)%]|h|r")
		name = Delocalizator:GetTranslation(itemString)
	end
	
	if (name) then
		if (spell == displayName) then 
			displayName = name 
		end
		spell = name
	end
	
	if (rank) then
		rank = gsub(rank, "Уровень", "Rank")
	end
	
	return spell, rank, displayName, icon, startTime, endTime, isTradeSkill, castID, interrupt
	
end

---------------------
-- Combat log hook --
---------------------
function Delocalizator:CombatLogHook()
	Delocalizator:HookFrames(GetFramesRegisteredForEvent('COMBAT_LOG_EVENT_UNFILTERED'))
	Delocalizator:HookFrames(GetFramesRegisteredForEvent('COMBAT_LOG_EVENT'))
end

local function EventHandler(self, event, ...)
local arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20 = ...
if (event == 'COMBAT_LOG_EVENT_UNFILTERED') or (event == 'COMBAT_LOG_EVENT') then
		if string.find(arg2, "SPELL") or string.find(arg2, "RANGE") or string.find(arg2, "DAMAGE_SHIELD") or string.find(arg2, "DAMAGE_SPLIT")then
			local engName
			if arg9 and arg10 then engName = Delocalizator.spelldata[tonumber(arg9)] end
			if engName then arg10 = engName end
			if     (arg20) then frame_handlers[self](self, event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20)
			elseif (arg19) then frame_handlers[self](self, event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19)
			elseif (arg18) then frame_handlers[self](self, event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18)
			elseif (arg17) then frame_handlers[self](self, event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17)
			elseif (arg16) then frame_handlers[self](self, event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16)
			elseif (arg15) then frame_handlers[self](self, event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15)
			elseif (arg14) then frame_handlers[self](self, event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14)
			elseif (arg13) then frame_handlers[self](self, event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13)
			elseif (arg12) then frame_handlers[self](self, event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12)
			elseif (arg11) then frame_handlers[self](self, event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11)
			elseif (arg10) then frame_handlers[self](self, event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10)
			else frame_handlers[self](self, event, ...) end
						
		else
			frame_handlers[self](self, event, ...)
		end
	else
		frame_handlers[self](self, event, ...)
	end
end

function Delocalizator:HookFrames(...)
	for i = 1 ,select('#', ...) do
		local frame = select(i, ...)
		if not frame_handlers[frame] and frame:GetScript('OnEvent') then
			frame_handlers[frame] = frame:GetScript('OnEvent')
			frame:SetScript('OnEvent', EventHandler)
		end
	end
end

---------------------
-- Chat frame hook --
---------------------
function Delocalizator:ChatFrame_OnEvent(this, event, arg1, ...)
	if (arg1) then arg1 = Delocalizator.StringToEnglish(arg1) end
	self.hooks["ChatFrame_OnEvent"](this, event, arg1, ...)
end

-------------------
-- Tooltips hook --
-------------------
function Delocalizator.ProcessItem(tooltip, name, link)
	local tipTextLeft = tooltip:GetName().."TextLeft"
	local fontString = _G[tipTextLeft..1]
	local text = fontString:GetText()

	if (link) and not dl_iskeyDown() then
		_, _, itemString = string.find(link, "^|c%x+|H(.+)|h%[")
		if (itemString) then fontString:SetText(Delocalizator:AddRuName(Delocalizator:GetTranslation(itemString), text)) end
	end
	
	tooltip:Show()
	
end

function Delocalizator.ProcessTalent(tooltip, tabIndex, talentIndex)
	local tipTextLeft = tooltip:GetName().."TextLeft"
	local fontString = _G[tipTextLeft..1]
	local text = fontString:GetText()
	local isInspect = false
	local isPet = false
	
	if (string.find(GetMouseFocus():GetName(),"Inspect") == 1) then isInspect = true end
	if (PlayerTalentFrame) then isPet = PlayerTalentFrame.pet end
	local talentLink = GetTalentLink(tabIndex, talentIndex, isInspect, isPet)
	
	if (talentLink) then
		_, _, talentString = string.find(talentLink, "^|c%x+|H(.+)|h%[")
		if (talentString) then 
			local _, _, talentId, talentRank = string.find(talentString, "^talent:(.+):(.+)")
			if talentRank == "-1" then talentRank = 0 end
			talentRank = talentRank + 1
			fontString:SetText(Delocalizator:AddRuName(Delocalizator.spelldata[tonumber(Delocalizator.talentdata[tonumber(talentId)][tonumber(talentRank)])], text)) 
		end
	end
	
	tooltip:Show()
	
end

function Delocalizator.ProcessSpell(tooltip, spellId, bookType)
	local tipTextLeft = tooltip:GetName().."TextLeft"
	local fontString = _G[tipTextLeft..1]
	local text = fontString:GetText()
	
	if (spellId == 0) then return end
	
	local spellLink, TradeSkillLink = GetSpellLink(spellId);
	
	if (spellLink) then
		_, _, spellString = string.find(spellLink, "^|c%x+|H(.+)|h%[")
		if (spellString) then fontString:SetText(Delocalizator:AddRuName(Delocalizator:GetTranslation(spellString), text)) end
	end
	
	tooltip:Show()
	
end

function Delocalizator.ProcessAction(tooltip, slot)
	local type, id, subtype = GetActionInfo(slot)
	
	if (type == "spell") then
		Delocalizator.ProcessSpell(tooltip, id, subtype)
	elseif (type == "item") then
		itemName, itemLink = GetItemInfo(id)
		Delocalizator.ProcessItem(tooltip, itemName, itemLink)
	end

end

--------------------
-- Core functions --
--------------------
function Delocalizator.StringToEnglish(text)

	if not text then return end

	local l = string.len(text)
	local k = 0
	local tmptext = text
	local link

	while (k < l) do
		_, k, link = string.find(text, linkPattern, k)
		if (link) then
			local _, _, itemString, itemName = string.find(link, "|c%x+|H(.-)|h%[(.-)%]|h|r")
			local engName = Delocalizator:GetTranslation(itemString)
			itemName = gsub(itemName, "\45", "\45\45")
			if engName then tmptext = string.gsub(tmptext, "|h%["..itemName.."%]|h|r", "|h["..engName.."]|h|r") end
		else
			k = l
		end
	end
	
	return tmptext
	
end

function Delocalizator:GetTranslation(itemString)
	if not itemString then return false end
	
	local _, _, idtype, id = string.find(itemString, "^(%a+):(%d+)")
	
	if (idtype == "talent") then
		idtype = "spell"
		local _, _, talentId, talentRank = string.find(itemString, "^talent:(.+):(.+)")
		if talentRank == "-1" then talentRank = 0 end
		talentRank = talentRank + 1
		local talentId = talentId.."-"..talentRank
		id = Delocalizator.talentdata[talentId]
	end

	return Delocalizator:GetNameById(id, idtype)
		
end

function Delocalizator:AddRuName(engName, ruName)
	
	if not engName then
		return ruName
	elseif (opt["tipformat"] == 1) then 
		return (engName.."\n|cff"..opt["tipcolor"].."("..ruName..")|r")
	elseif (opt["tipformat"] == 2) then
		return (ruName.."\n|cff"..opt["tipcolor"].."("..engName..")|r")
	else
		return engName
	end
	
end

function Delocalizator:GetNameById(id, idtype)
	id = tonumber(id)
	if (idtype == "item") then
		return Delocalizator.itemdata[id]
	elseif (idtype == "spell") or (idtype == "trade") or (idtype == "enchant") then
		return Delocalizator.spelldata[id]
	else
		return nil
	end

end
